/* This is the header file that contains the parameters for 
   kaon photoproduction at high s. */

#ifndef t_kaonprod_regge_h
#define t_kaonprod_regge_h



                          
   

#define ELEC 0.302862     
                          /* electric charge e */
			  /* with pow(e, 2) / (4 * PI) = 1 / 137 */
                        
/************************************************************************/
/*                                                                      */ 
/*                            KAON                                      */
/*                                                                      */ 
/************************************************************************/


#define K_M 0.4937
                          /* charged kaon mass (in GeV) */
                          /* 0.4937 */
#define K0_M 0.4977        
                          /* neutral kaon mass (in GeV) */
                          /* 0.4977 */
#define KM_M K_M



#define ALPHAp_K 0.7      /* 0.7 */ 


#define G_KNLAM -11.54     /* -11.54 */
                          /* coupling constant f_knlambda */
			  /* with pow(f_knlambda, 2) / (4 * PI) = 10.6 */

#define G_KNSIG 4.48      /* 4.48 */ 
                          /* coupling constant f_knsigma */
			  /* with pow(f_knsigma, 2) / (4 * PI) = 1.6 */
                         

#define PSPV_PARAM 0      /* 0 = Pseudo-vector coupling */
                          /* 1 = Pseudo-scalar coupling */ 


/************************************************************************/
/*                                                                      */ 
/*                         K_STAR MESON                                 */
/*                                                                      */ 
/************************************************************************/


#define KSTAR_M 0.892      
                          /* K_star meson mass in GeV */

#define ALPHAp_KSTAR 0.83    
                          /* 0.83 */ 

#define G_KSTARNLAM -23.       
                          /* vector coupling constant g_kstarnlambda */
#define G_KSTARNSIG -25.       
                          /* vector coupling constant g_kstarnsigma */
			    
#define KAPPA_KSTARLAM 2.5     
                          /* tensor to vector coupling constant for kstar  */
#define KAPPA_KSTARSIG -1.     
                          /* tensor to vector coupling constant for kstar  */
                          
#define G_KSTARKGA 0.842
/*#define G_KSTARKGA 0.892*/
                          /* coupling constant g_kstar-k-gamma */
                          /* definition, see our NPA, formula (25) */
                          /* 0.842 */



                                       
/************************************************************************/
/*                                                                      */ 
/*                            NUCLEON                                   */
/*                                                                      */ 
/************************************************************************/

#define NU_M 0.939         /* nucleon mass in GeV */

#define ALPHAp_N 0.97      /* 0.97 */ 

#define P_MAGN 2.79       
                          /* magnetic moment of proton */

#define F1_p 1.
#define F2_p 1.79
                          /* 1.79 */

#define N_MAGN (- 1.91)   
                          /* magnetic moment of neutron */

#define LAM_MNN 1.51      
                          /* cut-off in the M-N-N vertex */


/************************************************************************/
/*                                                                      */ 
/*                            LAMBDA (1115.7)                           */
/*                                                                      */ 
/************************************************************************/

#define LAM_M 1.1157       /* Lambda mass in GeV */

#define ALPHAp_LAM 0.9    

#define KAPPA_LAMLAM -0.73
#define KAPPA_LAMSIG 1.59

#define LAM_MNLAM 151.      
                          /* cut-off in the M-N-LAM vertex */


/************************************************************************/
/*                                                                      */ 
/*                            SIGMA (1193)                              */
/*                                                                      */ 
/************************************************************************/

#define SIG_M 1.1926      /* Sigma mass in GeV */

#define ALPHAp_SIG 0.85    

#define KAPPA_SIGSIG 1.

#define LAM_MNSIG 151.      
                          /* cut-off in the M-N-SIG vertex */



/************************************************************************/
/*                                                                      */ 
/*                            LAMBDA_STAR (1520)                        */
/*                                                                      */ 
/************************************************************************/

#define LAMS_M 1.520      /* Lambda_star mass in GeV */

/*#define G_KNLAMS 21.4     21.4  
                           coupling constant g_knlambda_star in GeV-1 */

#define G_KNLST 21.4     /* 21.4 */ 
                          /* coupling constant g_knlambda_star in GeV-1 */

/*#define G1_KSTNLST 32.*/     /* 21.4 */ 
                          /* coupling constant g_knlambda_star in GeV-1 */

/************************************************************************/
/*                                                                      */ 
/*                            P_33 (1232) RESONANCE                     */
/*                                                                      */ 
/************************************************************************/


#define DEL_M 1.232

#define ALPHAp_DEL 0.9    /* 0.9 */ 

#define M_p33 1.21876
                          /* delta mass in GeV */
                          /* 1.21876 relativistic, unitarized background
                                     for charged pion channels */  

#define F_PINDEL 1.949
                           /* coupling constant f_pindelta */
                           /* 1.949 unitarized background for
                                    charged pion channels */
                           
#define GM_p33 2.80        /* 2.80 */     
#define GE_p33 0.04        /* 0.04 */     

#define LAM_MNDEL 1.51      /* 1.5 */
                           /* cut-off in the M-N-Delta vertex */


#endif

























